Unit DirTransversor;
{ $Define Debug}
{*******************************************************}
{                                                       }
{       ML component bibliotek                          }
{       Directory gennemlbskomponent                   }
{       TDirTransver                                    }
{                                                       }
{       Copyright (c) 2000 Mads Lundwall/DF             }
{                                                       }
{*******************************************************}
{
  Et kald til "Koer" gennemlber directoryet sat i
  "RodDir" recursivt og returnerer alle filer der
  matcher "Maske" med et kald til funktionen sat med
  SaetBF indtil denne funktion returnerer False
  eller der ikke er flere filer.

  "Maske" kan vre et eller flere masker adskilt med ;
  feks. "*.pas;*.dfm"

  "RodDir" og "Maske" defaulter til hhv. 'C:\' og '*.*'

  Returnerer False hvis SaetBF-funktionen standsede
  krslen ved at returnere False, ellers returneres
  True.

  Kan rejse en exception af typen TDirTraversException
  hvis der ikke er sat en tilbagekalds-funktion med SaetBF.

}
{$Ifdef Debug}
  {$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V-,W-,X+,Y+,Z1}
{$else}
  {$A+,B-,C-,D-,E-,F-,G+,H+,I+,J+,K-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U+,V+,W-,X+,Y-,Z1}
{$Endif}
Interface

uses
  Windows, SysUtils, Classes, Forms, StdCtrls;

Type
  TDirTraversException = Exception;

  BehandlFunk = Function(Fil : String) : Boolean;

Type
  TDirTransver = class(TComponent)
  Private
    RodDirStart : String;
    Mask        : String;
    BehandlF    : BehandlFunk;
    AktSti      : TLabel;
    Function      LaesRodDir : String;
    Procedure     SaetRodDir(s : String);
    Procedure     Init;
  Public
    Stop        : Boolean;
    Constructor   Create(AOwner : TComponent); reintroduce;
    Function      Koer(KunUnderMapper : Boolean) : Boolean;
    Procedure     SaetBF(B : BehandlFunk);
  Published
    Property      Maske   : String Read Mask Write Mask; { Maske "*.txt" }
    Property      RodDir  : String Read LaesRodDir Write SaetRodDir; { Herfra gennemlbes tret }
    Property      AktuelSti : TLabel Read AktSti Write AktSti;
  End;

  Procedure Register;

Implementation
Uses
  Util;

  Procedure TDirTransver.Init;
  Begin
    RodDirStart:= '';
    Mask       := '';
    BehandlF   := Nil;
  End;

  Function  TDirTransver.LaesRodDir : String;
  Begin
    LaesRodDir:=RodDirStart
  End;

  Procedure TDirTransver.SaetRodDir(s : String);
  Begin
    If DirExists(s) Then
    Begin
      RodDirStart:=s;
      If Assigned(AktSti) Then
        AktSti.Caption:=s;
    End
    Else
      Raise(TDirTraversException.Create('Rodir '+s+' findes ikke'));
  End;

  Constructor TDirTransver.Create(AOwner : TComponent);
  Begin
    Init;
    TComponent.Create(AOwner)
  End;

  Procedure TDirTransver.SaetBF(B : BehandlFunk);
  Begin
    BehandlF:=B
  End;

  Function TDirTransver.Koer(KunUnderMapper : Boolean) : Boolean;
  Var
    p           : Word;
    IkkeHer     : Boolean;
    Msk,
    AktuelMaske : String;

    Procedure GnmLoebDir(D : String);
    Var
      SRec : TSearchRec;
    Begin
      Application.ProcessMessages;
      If Assigned(AktSti) Then
        AktSti.Caption:=TilSti(D);
      If (Not IkkeHer) And (FindFirst(TilSti(D)+AktuelMaske,faAnyFile,SRec)=0) Then
      Repeat
        If (Srec.Attr<>FaDirectory) Then
          Stop:=Not BehandlF(TilSti(D)+SRec.Name)
      Until (Stop) Or (FindNext(SRec)<>0);
      If Not Stop Then
      Begin
        IkkeHer:=False;
        If FindFirst(TilSti(D)+'*.*',faDirectory,SRec)=0 Then
        Repeat
          If (SRec.Name<>'.') And (SRec.Name<>'..') And ((Srec.Attr And FaDirectory)>0) Then
            GnmLoebDir(TilSti(D)+SRec.Name)
        Until Stop or (FindNext(SRec)<>0);
      End;
      FindClose(SRec);
    End;

  Begin
    If Not Assigned(BehandlF) Then
      Raise(TDirTraversException.Create('Tilbagekald udefineret'));
    If RodDirStart='' Then
      RodDirStart:='C:';

    Stop:=False;
    If Mask='' Then
      Msk:='*.*';
    Msk:=Mask+';';
    p:=Pos(';',Msk);
    While (p>1) Do
    Begin
      AktuelMaske:=Copy(Msk,1,Pred(p));
      Delete(Msk,1,p);
      IkkeHer:=KunUnderMapper;
      GnmLoebDir(RodDirStart);
      p:=Pos(';',Msk);
    End;
    Koer:=Not Stop
  End;

  Procedure Register;
  Begin
    RegisterComponents('ML', [TDirTransVer]);
  End;

End.
